@@ -8,7 +8,9 @@ from rasa.cli import SubParsersAction
 from rasa.cli.arguments import interactive as arguments
 import rasa.cli.train as train
 import rasa.cli.utils
+from rasa.engine.storage.local_model_storage import LocalModelStorage
 from rasa.shared.constants import DEFAULT_ENDPOINTS_PATH, DEFAULT_MODELS_PATH
+from rasa.shared.importers.autoconfig import TrainingType
 from rasa.shared.importers.importer import TrainingDataImporter
 import rasa.shared.utils.cli
 import rasa.utils.common
@@ -42,7 +44,7 @@ def add_subparser(
         parents=parents,
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
         help="Starts an interactive learning session model to create new training data "
-        "for a Rasa Core model by chatting. Uses the 'RegexInterpreter', i.e. "
+        "for a Rasa Core model by chatting. Uses the 'RegexMessageHandler', i.e. "
         "`/<intent>` input format.",
     )
     interactive_core_parser.set_defaults(func=interactive, core_only=True)
@@ -58,7 +60,7 @@ def interactive(args: argparse.Namespace) -> None:
     )
 
     if args.model is None:
-        story_graph = rasa.utils.common.run_in_loop(file_importer.get_stories())
+        story_graph = file_importer.get_stories()
         if not story_graph or story_graph.is_empty():
             rasa.shared.utils.cli.print_error_and_exit(
                 "Could not run interactive learning without either core "
@@ -110,21 +112,21 @@ def perform_interactive_learning(
 
     args.model = zipped_model
 
-    with model.unpack_model(zipped_model) as model_path:
-        args.core, args.nlu = model.get_model_subdirectories(model_path)
-        if args.core is None:
-            rasa.shared.utils.cli.print_error_and_exit(
-                "Can not run interactive learning on an NLU-only model."
-            )
-
-        args.endpoints = rasa.cli.utils.get_validated_path(
-            args.endpoints, "endpoints", DEFAULT_ENDPOINTS_PATH, True
+    metadata = LocalModelStorage.metadata_from_archive(zipped_model)
+    if metadata.training_type == TrainingType.NLU:
+        rasa.shared.utils.cli.print_error_and_exit(
+            "Can not run interactive learning on an NLU-only model."
         )
 
-        do_interactive_learning(args, file_importer)
+    args.endpoints = rasa.cli.utils.get_validated_path(
+        args.endpoints, "endpoints", DEFAULT_ENDPOINTS_PATH, True
+    )
+
+    do_interactive_learning(args, file_importer)
 
 
 def get_provided_model(arg_model: Text) -> Optional[Text]:
+    """Checks model path input and selects model from it."""
     model_path = rasa.cli.utils.get_validated_path(
         arg_model, "model", DEFAULT_MODELS_PATH
     )
