@@ -7,6 +7,9 @@ from typing import List
 from rasa import telemetry
 from rasa.cli import SubParsersAction
 from rasa.cli.arguments import shell as arguments
+from rasa.engine.storage.local_model_storage import LocalModelStorage
+from rasa.model import get_latest_model
+from rasa.shared.importers.autoconfig import TrainingType
 from rasa.shared.utils.cli import print_error
 from rasa.exceptions import ModelNotFound
 
@@ -58,9 +61,9 @@ def add_subparser(
 
 
 def shell_nlu(args: argparse.Namespace) -> None:
+    """Talk with an NLU only bot though the command line."""
     from rasa.cli.utils import get_validated_path
     from rasa.shared.constants import DEFAULT_MODELS_PATH
-    from rasa.model import get_model, get_model_subdirectories
     import rasa.nlu.run
 
     args.connector = "cmdline"
@@ -68,7 +71,7 @@ def shell_nlu(args: argparse.Namespace) -> None:
     model = get_validated_path(args.model, "model", DEFAULT_MODELS_PATH)
 
     try:
-        model_path = get_model(model)
+        model = get_latest_model(model)
     except ModelNotFound:
         print_error(
             "No model found. Train a model before running the "
@@ -76,9 +79,8 @@ def shell_nlu(args: argparse.Namespace) -> None:
         )
         return
 
-    _, nlu_model = get_model_subdirectories(model_path)
-
-    if not nlu_model:
+    metadata = LocalModelStorage.metadata_from_archive(model)
+    if metadata.training_type == TrainingType.CORE:
         print_error(
             "No NLU model found. Train a model before running the "
             "server using `rasa train nlu`."
@@ -86,20 +88,20 @@ def shell_nlu(args: argparse.Namespace) -> None:
         return
 
     telemetry.track_shell_started("nlu")
-    rasa.nlu.run.run_cmdline(nlu_model)
+    rasa.nlu.run.run_cmdline(model)
 
 
 def shell(args: argparse.Namespace) -> None:
+    """Talk with a bot though the command line."""
     from rasa.cli.utils import get_validated_path
     from rasa.shared.constants import DEFAULT_MODELS_PATH
-    from rasa.model import get_model, get_model_subdirectories
 
     args.connector = "cmdline"
 
     model = get_validated_path(args.model, "model", DEFAULT_MODELS_PATH)
 
     try:
-        model_path = get_model(model)
+        model = get_latest_model(model)
     except ModelNotFound:
         print_error(
             "No model found. Train a model before running the "
@@ -107,14 +109,14 @@ def shell(args: argparse.Namespace) -> None:
         )
         return
 
-    core_model, nlu_model = get_model_subdirectories(model_path)
+    metadata = LocalModelStorage.metadata_from_archive(model)
 
-    if not core_model:
+    if metadata.training_type == TrainingType.NLU:
         import rasa.nlu.run
 
         telemetry.track_shell_started("nlu")
 
-        rasa.nlu.run.run_cmdline(nlu_model)
+        rasa.nlu.run.run_cmdline(model)
     else:
         import rasa.cli.run
 
