@@ -2,7 +2,6 @@ import asyncio
 import logging
 import uuid
 import os
-import shutil
 from functools import partial
 from typing import Any, List, Optional, Text, Union, Dict
 
@@ -12,16 +11,12 @@ import rasa.shared.utils.common
 import rasa.utils
 import rasa.utils.common
 import rasa.utils.io
-from rasa import model, server, telemetry
+from rasa import server, telemetry
 from rasa.constants import ENV_SANIC_BACKLOG
 from rasa.core import agent, channels, constants
 from rasa.core.agent import Agent
-from rasa.core.brokers.broker import EventBroker
 from rasa.core.channels import console
 from rasa.core.channels.channel import InputChannel
-import rasa.core.interpreter
-from rasa.core.lock_store import LockStore
-from rasa.core.tracker_store import TrackerStore
 from rasa.core.utils import AvailableEndpoints
 import rasa.shared.utils.io
 from sanic import Sanic
@@ -34,7 +29,6 @@ def create_http_input_channels(
     channel: Optional[Text], credentials_file: Optional[Text]
 ) -> List["InputChannel"]:
     """Instantiate the chosen input channel."""
-
     if credentials_file:
         all_credentials = rasa.shared.utils.io.read_config_file(credentials_file)
     else:
@@ -76,7 +70,7 @@ def _create_single_channel(channel: Text, credentials: Dict[Text, Any]) -> Any:
 
 
 def _create_app_without_api(cors: Optional[Union[Text, List[Text]]] = None) -> Sanic:
-    app = Sanic("rasa_core_no_api", configure_logging=False, register=False)
+    app = Sanic(__name__, configure_logging=False)
     server.add_root_route(app)
     server.configure_cors(app, cors)
     return app
@@ -95,10 +89,15 @@ def configure_app(
     endpoints: Optional[AvailableEndpoints] = None,
     log_file: Optional[Text] = None,
     conversation_id: Optional[Text] = uuid.uuid4().hex,
+    use_syslog: bool = False,
+    syslog_address: Optional[Text] = None,
+    syslog_port: Optional[int] = None,
+    syslog_protocol: Optional[Text] = None,
 ) -> Sanic:
     """Run the agent."""
-
-    rasa.core.utils.configure_file_logging(logger, log_file)
+    rasa.core.utils.configure_file_logging(
+        logger, log_file, use_syslog, syslog_address, syslog_port, syslog_protocol,
+    )
 
     if enable_api:
         app = server.create_app(
@@ -148,6 +147,7 @@ def configure_app(
 def serve_application(
     model_path: Optional[Text] = None,
     channel: Optional[Text] = None,
+    interface: Optional[Text] = constants.DEFAULT_SERVER_INTERFACE,
     port: int = constants.DEFAULT_SERVER_PORT,
     credentials: Optional[Text] = None,
     cors: Optional[Union[Text, List[Text]]] = None,
@@ -164,9 +164,12 @@ def serve_application(
     ssl_ca_file: Optional[Text] = None,
     ssl_password: Optional[Text] = None,
     conversation_id: Optional[Text] = uuid.uuid4().hex,
+    use_syslog: Optional[bool] = False,
+    syslog_address: Optional[Text] = None,
+    syslog_port: Optional[int] = None,
+    syslog_protocol: Optional[Text] = None,
 ) -> None:
     """Run the API entrypoint."""
-
     if not channel and not credentials:
         channel = "cmdline"
 
@@ -184,6 +187,10 @@ def serve_application(
         endpoints=endpoints,
         log_file=log_file,
         conversation_id=conversation_id,
+        use_syslog=use_syslog,
+        syslog_address=syslog_address,
+        syslog_port=syslog_port,
+        syslog_protocol=syslog_protocol,
     )
 
     ssl_context = server.create_ssl_context(
@@ -191,10 +198,7 @@ def serve_application(
     )
     protocol = "https" if ssl_context else "http"
 
-    logger.info(
-        f"Starting Rasa server on "
-        f"{constants.DEFAULT_SERVER_FORMAT.format(protocol, port)}"
-    )
+    logger.info(f"Starting Rasa server on {protocol}://{interface}:{port}")
 
     app.register_listener(
         partial(load_agent_on_start, model_path, endpoints, remote_storage),
@@ -202,11 +206,6 @@ def serve_application(
     )
     app.register_listener(close_resources, "after_server_stop")
 
-    # noinspection PyUnresolvedReferences
-    async def clear_model_files(_app: Sanic, _loop: Text) -> None:
-        if app.ctx.agent.model_directory:
-            shutil.rmtree(_app.ctx.agent.model_directory)
-
     number_of_workers = rasa.core.utils.number_of_sanic_workers(
         endpoints.lock_store if endpoints else None
     )
@@ -215,11 +214,12 @@ def serve_application(
         input_channels, endpoints, model_path, number_of_workers, enable_api
     )
 
-    app.register_listener(clear_model_files, "after_server_stop")
+    rasa.utils.common.update_sanic_log_level(
+        log_file, use_syslog, syslog_address, syslog_port, syslog_protocol,
+    )
 
-    rasa.utils.common.update_sanic_log_level(log_file)
     app.run(
-        host="0.0.0.0",
+        host=interface,
         port=port,
         ssl=ssl_context,
         backlog=int(os.environ.get(ENV_SANIC_BACKLOG, "100")),
@@ -240,57 +240,14 @@ async def load_agent_on_start(
     Used to be scheduled on server start
     (hence the `app` and `loop` arguments).
     """
-    # noinspection PyBroadException
-    try:
-        with model.get_model(model_path) as unpacked_model:
-            _, nlu_model = model.get_model_subdirectories(unpacked_model)
-            _interpreter = rasa.core.interpreter.create_interpreter(
-                endpoints.nlu or nlu_model
-            )
-    except Exception:
-        logger.debug(f"Could not load interpreter from '{model_path}'.")
-        _interpreter = None
-
-    _broker = await EventBroker.create(endpoints.event_broker, loop=loop)
-    _tracker_store = TrackerStore.create(endpoints.tracker_store, event_broker=_broker)
-    _lock_store = LockStore.create(endpoints.lock_store)
-
-    model_server = endpoints.model if endpoints and endpoints.model else None
-
-    try:
-        app.ctx.agent = await agent.load_agent(
-            model_path,
-            model_server=model_server,
-            remote_storage=remote_storage,
-            interpreter=_interpreter,
-            generator=endpoints.nlg,
-            tracker_store=_tracker_store,
-            lock_store=_lock_store,
-            action_endpoint=endpoints.action,
-        )
-    except Exception as e:
-        rasa.shared.utils.io.raise_warning(
-            f"The model at '{model_path}' could not be loaded. "
-            f"Error: {type(e)}: {e}"
-        )
-        app.ctx.agent = None
-
-    if not app.ctx.agent:
-        rasa.shared.utils.io.raise_warning(
-            "Agent could not be loaded with the provided configuration. "
-            "Load default agent without any model."
-        )
-        app.ctx.agent = Agent(
-            interpreter=_interpreter,
-            generator=endpoints.nlg,
-            tracker_store=_tracker_store,
-            action_endpoint=endpoints.action,
-            model_server=model_server,
-            remote_storage=remote_storage,
-        )
-
+    app.agent = await agent.load_agent(
+        model_path=model_path,
+        remote_storage=remote_storage,
+        endpoints=endpoints,
+        loop=loop,
+    )
     logger.info("Rasa server is up and running.")
-    return app.ctx.agent
+    return app.agent
 
 
 async def close_resources(app: Sanic, _: AbstractEventLoop) -> None:
@@ -300,20 +257,11 @@ async def close_resources(app: Sanic, _: AbstractEventLoop) -> None:
         app: The Sanic application.
         _: The current Sanic worker event loop.
     """
-    current_agent = getattr(app.ctx, "agent", None)
+    current_agent = getattr(app, "agent", None)
     if not current_agent:
         logger.debug("No agent found when shutting down server.")
         return
 
     event_broker = current_agent.tracker_store.event_broker
     if event_broker:
-        if not asyncio.iscoroutinefunction(event_broker.close):
-            rasa.shared.utils.io.raise_deprecation_warning(
-                f"The method '{EventBroker.__name__}.{EventBroker.close.__name__}' was "
-                f"changed to be asynchronous. Please adapt your custom event broker "
-                f"accordingly. Support for synchronous implementations will be removed "
-                f"in Rasa Open Source 3.0.0."
-            )
-            event_broker.close()
-        else:
-            await event_broker.close()
+        await event_broker.close()
