@@ -1,36 +1,33 @@
+from __future__ import annotations
 import numpy as np
 import logging
 
-from typing import Any, Optional, Text, List, Type, Dict, Tuple
+from typing import Any, Text, List, Dict, Tuple, Type
 
-import rasa.core.utils
-from rasa.nlu.config import RasaNLUModelConfig
-from rasa.nlu.components import Component, UnsupportedLanguageError
-from rasa.nlu.featurizers.featurizer import DenseFeaturizer
-from rasa.nlu.model import Metadata
-import rasa.shared.utils.io
-from rasa.shared.nlu.training_data.features import Features
-from rasa.nlu.tokenizers.tokenizer import Tokenizer, Token
+from rasa.engine.graph import ExecutionContext, GraphComponent
+from rasa.engine.recipes.default_recipe import DefaultV1Recipe
+from rasa.engine.storage.resource import Resource
+from rasa.engine.storage.storage import ModelStorage
+from rasa.nlu.featurizers.dense_featurizer.dense_featurizer import DenseFeaturizer
+from rasa.nlu.tokenizers.tokenizer import Token, Tokenizer
 from rasa.shared.nlu.training_data.training_data import TrainingData
 from rasa.shared.nlu.training_data.message import Message
 from rasa.nlu.constants import (
     DENSE_FEATURIZABLE_ATTRIBUTES,
     SEQUENCE_FEATURES,
     SENTENCE_FEATURES,
-    FEATURIZER_CLASS_ALIAS,
     NO_LENGTH_RESTRICTION,
     NUMBER_OF_SUB_TOKENS,
     TOKENS_NAMES,
-    LANGUAGE_MODEL_DOCS,
 )
 from rasa.shared.nlu.constants import (
     TEXT,
-    FEATURE_TYPE_SENTENCE,
-    FEATURE_TYPE_SEQUENCE,
     ACTION_TEXT,
 )
 from rasa.utils import train_utils
 
+logger = logging.getLogger(__name__)
+
 MAX_SEQUENCE_LENGTHS = {
     "bert": 512,
     "gpt": 512,
@@ -40,110 +37,75 @@ MAX_SEQUENCE_LENGTHS = {
     "roberta": 512,
 }
 
-logger = logging.getLogger(__name__)
 
+@DefaultV1Recipe.register(
+    DefaultV1Recipe.ComponentType.MESSAGE_FEATURIZER, is_trainable=False
+)
+class LanguageModelFeaturizer(DenseFeaturizer, GraphComponent):
+    """A featurizer that uses transformer-based language models.
 
-class LanguageModelFeaturizer(DenseFeaturizer):
-    """Featurizer using transformer-based language models.
-
-    The transformers(https://github.com/huggingface/transformers) library
-    is used to load pre-trained language models like BERT, GPT-2, etc.
-    The component also tokenizes and featurizes dense featurizable attributes of
+    This component loads a pre-trained language model
+    from the Transformers library (https://github.com/huggingface/transformers)
+    including BERT, GPT, GPT-2, xlnet, distilbert, and roberta.
+    It also tokenizes and featurizes the featurizable dense attributes of
     each message.
     """
 
-    defaults = {
-        # name of the language model to load.
-        "model_name": "bert",
-        # Pre-Trained weights to be loaded(string)
-        "model_weights": None,
-        # an optional path to a specific directory to download
-        # and cache the pre-trained model weights.
-        "cache_dir": None,
-    }
-
     @classmethod
-    def required_components(cls) -> List[Type[Component]]:
-        """Packages needed to be installed."""
+    def required_components(cls) -> List[Type]:
+        """Components that should be included in the pipeline before this component."""
         return [Tokenizer]
 
     def __init__(
-        self,
-        component_config: Optional[Dict[Text, Any]] = None,
-        skip_model_load: bool = False,
-        hf_transformers_loaded: bool = False,
+        self, config: Dict[Text, Any], execution_context: ExecutionContext,
     ) -> None:
-        """Initializes LanguageModelFeaturizer with the specified model.
-
-        Args:
-            component_config: Configuration for the component.
-            skip_model_load: Skip loading the model for pytests.
-            hf_transformers_loaded: Skip loading of model and metadata, use
-            HFTransformers output instead.
-        """
-        super(LanguageModelFeaturizer, self).__init__(component_config)
-        if hf_transformers_loaded:
-            return
+        """Initializes the featurizer with the model in the config."""
+        super(LanguageModelFeaturizer, self).__init__(
+            execution_context.node_name, config
+        )
         self._load_model_metadata()
-        self._load_model_instance(skip_model_load)
+        self._load_model_instance()
+
+    @staticmethod
+    def get_default_config() -> Dict[Text, Any]:
+        """Returns LanguageModelFeaturizer's default config."""
+        return {
+            **DenseFeaturizer.get_default_config(),
+            # name of the language model to load.
+            "model_name": "bert",
+            # Pre-Trained weights to be loaded(string)
+            "model_weights": None,
+            # an optional path to a specific directory to download
+            # and cache the pre-trained model weights.
+            "cache_dir": None,
+        }
+
+    @classmethod
+    def validate_config(cls, config: Dict[Text, Any]) -> None:
+        """Validates the configuration."""
+        pass
 
     @classmethod
     def create(
-        cls, component_config: Dict[Text, Any], config: RasaNLUModelConfig
-    ) -> "DenseFeaturizer":
-        language = config.language
-        if not cls.can_handle_language(language):
-            # check failed
-            raise UnsupportedLanguageError(cls.name, language)
-        # TODO: remove this when HFTransformersNLP is removed for good
-        if isinstance(config, Metadata):
-            hf_transformers_loaded = "HFTransformersNLP" in [
-                c["name"] for c in config.metadata["pipeline"]
-            ]
-        else:
-            hf_transformers_loaded = "HFTransformersNLP" in config.component_names
-        return cls(component_config, hf_transformers_loaded=hf_transformers_loaded)
-
-    @classmethod
-    def load(
         cls,
-        meta: Dict[Text, Any],
-        model_dir: Text,
-        model_metadata: Optional["Metadata"] = None,
-        cached_component: Optional["Component"] = None,
-        **kwargs: Any,
-    ) -> "Component":
-        """Load this component from file.
+        config: Dict[Text, Any],
+        model_storage: ModelStorage,
+        resource: Resource,
+        execution_context: ExecutionContext,
+    ) -> LanguageModelFeaturizer:
+        """Creates a LanguageModelFeaturizer.
 
-        After a component has been trained, it will be persisted by
-        calling `persist`. When the pipeline gets loaded again,
-        this component needs to be able to restore itself.
-        Components can rely on any context attributes that are
-        created by :meth:`components.Component.create`
-        calls to components previous to this one.
-
-        This method differs from the parent method only in that it calls create
-        rather than the constructor if the component is not found. This is to
-        trigger the check for HFTransformersNLP and the method can be removed
-        when HFTRansformersNLP is removed.
-
-        Args:
-                meta: Any configuration parameter related to the model.
-                model_dir: The directory to load the component from.
-                model_metadata: The model's :class:`rasa.nlu.model.Metadata`.
-                cached_component: The cached component.
-
-        Returns:
-                the loaded component
+        Loads the model specified in the config.
         """
-        # TODO: remove this when HFTransformersNLP is removed for good
-        if cached_component:
-            return cached_component
+        return cls(config, execution_context)
 
-        return cls.create(meta, model_metadata)
+    @staticmethod
+    def required_packages() -> List[Text]:
+        """Returns the extra python dependencies required."""
+        return ["transformers"]
 
     def _load_model_metadata(self) -> None:
-        """Load the metadata for the specified model and sets these properties.
+        """Loads the metadata for the specified model and set them as properties.
 
         This includes the model name, model weights, cache directory and the
         maximum sequence length the model can handle.
@@ -153,7 +115,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
             model_weights_defaults,
         )
 
-        self.model_name = self.component_config["model_name"]
+        self.model_name = self._config["model_name"]
 
         if self.model_name not in model_class_dict:
             raise KeyError(
@@ -162,8 +124,8 @@ class LanguageModelFeaturizer(DenseFeaturizer):
                 f"a new class inheriting from this class to support your model."
             )
 
-        self.model_weights = self.component_config["model_weights"]
-        self.cache_dir = self.component_config["cache_dir"]
+        self.model_weights = self._config["model_weights"]
+        self.cache_dir = self._config["cache_dir"]
 
         if not self.model_weights:
             logger.info(
@@ -174,17 +136,12 @@ class LanguageModelFeaturizer(DenseFeaturizer):
 
         self.max_model_sequence_length = MAX_SEQUENCE_LENGTHS[self.model_name]
 
-    def _load_model_instance(self, skip_model_load: bool) -> None:
-        """Try loading the model instance.
+    def _load_model_instance(self) -> None:
+        """Tries to load the model instance.
 
-        Args:
-            skip_model_load: Skip loading the model instances to save time. This
-            should be True only for pytests
+        Model loading should be skipped in unit tests.
+        See unit tests for examples.
         """
-        if skip_model_load:
-            # This should be True only during pytests
-            return
-
         from rasa.nlu.utils.hugging_face.registry import (
             model_class_dict,
             model_tokenizer_dict,
@@ -207,32 +164,8 @@ class LanguageModelFeaturizer(DenseFeaturizer):
         # while feeding input.
         self.pad_token_id = self.tokenizer.unk_token_id
 
-    @classmethod
-    def cache_key(
-        cls, component_meta: Dict[Text, Any], model_metadata: Metadata
-    ) -> Optional[Text]:
-        """Cache the component for future use.
-
-        Args:
-            component_meta: configuration for the component.
-            model_metadata: configuration for the whole pipeline.
-
-        Returns: key of the cache for future retrievals.
-        """
-        weights = component_meta.get("model_weights") or {}
-
-        return (
-            f"{cls.name}-{component_meta.get('model_name')}-"
-            f"{rasa.shared.utils.io.deep_container_fingerprint(weights)}"
-        )
-
-    @classmethod
-    def required_packages(cls) -> List[Text]:
-        """Packages needed to be installed."""
-        return ["transformers"]
-
     def _lm_tokenize(self, text: Text) -> Tuple[List[int], List[Text]]:
-        """Pass the text through the tokenizer of the language model.
+        """Passes the text through the tokenizer of the language model.
 
         Args:
             text: Text to be tokenized.
@@ -248,8 +181,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
     def _add_lm_specific_special_tokens(
         self, token_ids: List[List[int]]
     ) -> List[List[int]]:
-        """Add language model specific special tokens which were used during
-        their training.
+        """Adds the language and model-specific tokens used during training.
 
         Args:
             token_ids: List of token ids for each example in the batch.
@@ -269,7 +201,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
     def _lm_specific_token_cleanup(
         self, split_token_ids: List[int], token_strings: List[Text]
     ) -> Tuple[List[int], List[Text]]:
-        """Clean up special chars added by tokenizers of language models.
+        """Cleans up special chars added by tokenizers of language models.
 
         Many language models add a special char in front/back of (some) words. We clean
         up those chars as they are not
@@ -290,7 +222,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
     def _post_process_sequence_embeddings(
         self, sequence_embeddings: np.ndarray
     ) -> Tuple[np.ndarray, np.ndarray]:
-        """Compute sentence and sequence level representations for relevant tokens.
+        """Computes sentence and sequence level representations for relevant tokens.
 
         Args:
             sequence_embeddings: Sequence level dense features received as output from
@@ -322,7 +254,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
     def _tokenize_example(
         self, message: Message, attribute: Text
     ) -> Tuple[List[Token], List[int]]:
-        """Tokenize a single message example.
+        """Tokenizes a single message example.
 
         Many language models add a special char in front of (some) words and split
         words into sub-words. To ensure the entity start and end values matches the
@@ -369,7 +301,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
     def _get_token_ids_for_batch(
         self, batch_examples: List[Message], attribute: Text
     ) -> Tuple[List[List[Token]], List[List[int]]]:
-        """Compute token ids and token strings for each example in batch.
+        """Computes token ids and token strings for each example in batch.
 
         A token id is the id of that token in the vocabulary of the language model.
 
@@ -397,7 +329,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
     def _compute_attention_mask(
         actual_sequence_lengths: List[int], max_input_sequence_length: int
     ) -> np.ndarray:
-        """Compute a mask for padding tokens.
+        """Computes a mask for padding tokens.
 
         This mask will be used by the language model so that it does not attend to
         padding tokens.
@@ -467,7 +399,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
     def _add_padding_to_batch(
         self, batch_token_ids: List[List[int]], max_sequence_length_model: int
     ) -> List[List[int]]:
-        """Add padding so that all examples in the batch are of the same length.
+        """Adds padding so that all examples in the batch are of the same length.
 
         Args:
             batch_token_ids: Batch of examples where each example is a non-padded list
@@ -501,7 +433,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
     def _extract_nonpadded_embeddings(
         embeddings: np.ndarray, actual_sequence_lengths: List[int]
     ) -> np.ndarray:
-        """Extract embeddings for actual tokens.
+        """Extracts embeddings for actual tokens.
 
         Use pre-computed non-padded lengths of each example to extract embeddings
         for non-padding tokens.
@@ -523,7 +455,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
     def _compute_batch_sequence_features(
         self, batch_attention_mask: np.ndarray, padded_token_ids: List[List[int]]
     ) -> np.ndarray:
-        """Feed the padded batch to the language model.
+        """Feeds the padded batch to the language model.
 
         Args:
             batch_attention_mask: Mask of 0s and 1s which indicate whether the token
@@ -551,10 +483,12 @@ class LanguageModelFeaturizer(DenseFeaturizer):
         attribute: Text,
         inference_mode: bool = False,
     ) -> None:
-        """Validate if sequence lengths of all inputs are less the max sequence
-        length the model can handle.
+        """Validates sequence length.
 
-        This method should throw an error during training, whereas log a debug
+        Checks if sequence lengths of inputs are less than
+        the max sequence length the model can handle.
+
+        This method should throw an error during training, and log a debug
         message during inference if any of the input examples have a length
         greater than maximum sequence length allowed.
 
@@ -562,7 +496,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
             actual_sequence_lengths: original sequence length of all inputs
             batch_examples: all message instances in the batch
             attribute: attribute of message object to be processed
-            inference_mode: Whether this is during training or during inferencing
+            inference_mode: whether this is during training or inference
         """
         if self.max_model_sequence_length == NO_LENGTH_RESTRICTION:
             # There is no restriction on sequence length from the model
@@ -590,7 +524,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
     def _add_extra_padding(
         self, sequence_embeddings: np.ndarray, actual_sequence_lengths: List[int]
     ) -> np.ndarray:
-        """Add extra zero padding to match the original sequence length.
+        """Adds extra zero padding to match the original sequence length.
 
         This is only done if the input was truncated during the batch
         preparation of input for the model.
@@ -635,7 +569,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
         attribute: Text,
         inference_mode: bool = False,
     ) -> Tuple[np.ndarray, np.ndarray]:
-        """Compute dense features of each example in the batch.
+        """Computes dense features of each example in the batch.
 
         We first add the special tokens corresponding to each language model. Next, we
         add appropriate padding and compute a mask for that padding so that it doesn't
@@ -731,7 +665,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
         attribute: Text,
         inference_mode: bool = False,
     ) -> List[Dict[Text, Any]]:
-        """Compute language model docs for all examples in the batch.
+        """Computes language model docs for all examples in the batch.
 
         Args:
             batch_examples: Batch of message objects for which language model docs
@@ -744,19 +678,6 @@ class LanguageModelFeaturizer(DenseFeaturizer):
         Returns:
             List of language model docs for each message in batch.
         """
-        hf_transformers_doc = batch_examples[0].get(LANGUAGE_MODEL_DOCS[attribute])
-        if hf_transformers_doc:
-            # This should only be the case if the deprecated
-            # HFTransformersNLP component is used in the pipeline
-            # TODO: remove this when HFTransformersNLP is removed for good
-            logging.debug(
-                f"'{LANGUAGE_MODEL_DOCS[attribute]}' set: this "
-                f"indicates you're using the deprecated component "
-                f"HFTransformersNLP, please remove it from your "
-                f"pipeline."
-            )
-            return [ex.get(LANGUAGE_MODEL_DOCS[attribute]) for ex in batch_examples]
-
         batch_tokens, batch_token_ids = self._get_token_ids_for_batch(
             batch_examples, attribute
         )
@@ -780,13 +701,8 @@ class LanguageModelFeaturizer(DenseFeaturizer):
 
         return batch_docs
 
-    def train(
-        self,
-        training_data: TrainingData,
-        config: Optional[RasaNLUModelConfig] = None,
-        **kwargs: Any,
-    ) -> None:
-        """Compute tokens and dense features for each message in training data.
+    def process_training_data(self, training_data: TrainingData,) -> TrainingData:
+        """Computes tokens and dense features for each message in training data.
 
         Args:
             training_data: NLU training data to be tokenized and featurized
@@ -818,15 +734,19 @@ class LanguageModelFeaturizer(DenseFeaturizer):
                     self._set_lm_features(batch_docs[index], ex, attribute)
                 batch_start_index += batch_size
 
-    def process(self, message: Message, **kwargs: Any) -> None:
-        """Process an incoming message by computing its tokens and dense features.
+        return training_data
 
-        Args:
-            message: Incoming message object
-        """
-        # process of all featurizers operates only on TEXT and ACTION_TEXT attributes,
-        # because all other attributes are labels which are featurized during training
-        # and their features are stored by the model itself.
+    def process(self, messages: List[Message]) -> List[Message]:
+        """Processes messages by computing tokens and dense features."""
+        for message in messages:
+            self._process_message(message)
+        return messages
+
+    def _process_message(self, message: Message) -> Message:
+        """Processes a message by computing tokens and dense features."""
+        # processing featurizers operates only on TEXT and ACTION_TEXT attributes,
+        # because all other attributes are labels which are featurized during
+        # training and their features are stored by the model itself.
         for attribute in {TEXT, ACTION_TEXT}:
             if message.get(attribute):
                 self._set_lm_features(
@@ -836,6 +756,7 @@ class LanguageModelFeaturizer(DenseFeaturizer):
                     message,
                     attribute,
                 )
+        return message
 
     def _set_lm_features(
         self, doc: Dict[Text, Any], message: Message, attribute: Text = TEXT
@@ -844,17 +765,9 @@ class LanguageModelFeaturizer(DenseFeaturizer):
         sequence_features = doc[SEQUENCE_FEATURES]
         sentence_features = doc[SENTENCE_FEATURES]
 
-        final_sequence_features = Features(
-            sequence_features,
-            FEATURE_TYPE_SEQUENCE,
-            attribute,
-            self.component_config[FEATURIZER_CLASS_ALIAS],
+        self.add_features_to_message(
+            sequence=sequence_features,
+            sentence=sentence_features,
+            attribute=attribute,
+            message=message,
         )
-        message.add_features(final_sequence_features)
-        final_sentence_features = Features(
-            sentence_features,
-            FEATURE_TYPE_SENTENCE,
-            attribute,
-            self.component_config[FEATURIZER_CLASS_ALIAS],
-        )
-        message.add_features(final_sentence_features)
