@@ -1,6 +1,13 @@
 from collections import OrderedDict
 from pathlib import Path
-from typing import Any, Dict, List, Text, Union, Optional
+from typing import (
+    Any,
+    Dict,
+    List,
+    Text,
+    Union,
+    Optional,
+)
 
 from ruamel import yaml
 from ruamel.yaml.comments import CommentedMap
@@ -94,7 +101,6 @@ class YAMLStoryWriter(StoryWriter):
         result = self.stories_to_yaml(story_steps, is_test_story)
         if is_appendable and KEY_STORIES in result:
             result = result[KEY_STORIES]
-
         rasa.shared.utils.io.write_yaml(result, target, True)
 
     def stories_to_yaml(
@@ -142,7 +148,6 @@ class YAMLStoryWriter(StoryWriter):
         result = OrderedDict()
         result[KEY_STORY_NAME] = story_step.block_name
         steps = self.process_checkpoints(story_step.start_checkpoints)
-
         for event in story_step.events:
             if not self._filter_event(event):
                 continue
@@ -203,29 +208,14 @@ class YAMLStoryWriter(StoryWriter):
         """
         result = CommentedMap()
         if user_utterance.intent_name and not user_utterance.use_text_for_featurization:
-            result[KEY_USER_INTENT] = user_utterance.intent_name
-
-        if hasattr(user_utterance, "inline_comment"):
-            comment = user_utterance.inline_comment()
-            if comment:
-                result.yaml_add_eol_comment(comment, KEY_USER_INTENT)
-
-        if user_utterance.text and (
-            # We only print the utterance text if it was an end-to-end prediction
-            user_utterance.use_text_for_featurization
-            # or if we want to print a conversation test story.
-            or is_test_story
-        ):
-            result[KEY_USER_MESSAGE] = LiteralScalarString(
-                rasa.shared.core.events.format_message(
-                    user_utterance.text,
-                    user_utterance.intent_name,
-                    user_utterance.entities,
-                )
+            result[KEY_USER_INTENT] = (
+                user_utterance.full_retrieval_intent_name
+                if user_utterance.full_retrieval_intent_name
+                else user_utterance.intent_name
             )
 
+        entities = []
         if len(user_utterance.entities) and not is_test_story:
-            entities = []
             for entity in user_utterance.entities:
                 if "value" in entity:
                     if hasattr(user_utterance, "inline_comment_for_entity"):
@@ -256,6 +246,27 @@ class YAMLStoryWriter(StoryWriter):
                     entities.append(entity["entity"])
             result[KEY_ENTITIES] = entities
 
+        if hasattr(user_utterance, "inline_comment"):
+            comment = user_utterance.inline_comment(
+                force_comment_generation=not entities
+            )
+            if comment:
+                result.yaml_add_eol_comment(comment, KEY_USER_INTENT)
+
+        if user_utterance.text and (
+            # We only print the utterance text if it was an end-to-end prediction
+            user_utterance.use_text_for_featurization
+            # or if we want to print a conversation test story.
+            or is_test_story
+        ):
+            result[KEY_USER_MESSAGE] = LiteralScalarString(
+                rasa.shared.core.events.format_message(
+                    user_utterance.text,
+                    user_utterance.intent_name,
+                    user_utterance.entities,
+                )
+            )
+
         return result
 
     @staticmethod
@@ -287,14 +298,14 @@ class YAMLStoryWriter(StoryWriter):
         return result
 
     @staticmethod
-    def process_slot(event: SlotSet) -> Dict[Text, List[Dict]]:
+    def process_slot(event: SlotSet) -> OrderedDict:
         """Converts a single `SlotSet` event into an ordered dict.
 
         Args:
             event: Original `SlotSet` event.
 
         Returns:
-            Dict with an `SlotSet` event.
+            OrderedDict with an `SlotSet` event.
         """
         return OrderedDict([(KEY_SLOT_NAME, [{event.key: event.value}])])
 
