@@ -4,10 +4,10 @@ import random
 from typing import Any, Text, List, Dict, Optional, TYPE_CHECKING, Set
 
 import rasa.shared.utils.io
+from rasa.shared.constants import INTENT_MESSAGE_PREFIX
 from rasa.shared.core.constants import ACTION_LISTEN_NAME
 from rasa.shared.core.domain import Domain
 from rasa.shared.core.events import UserUttered, ActionExecuted, Event
-from rasa.shared.nlu.interpreter import NaturalLanguageInterpreter, RegexInterpreter
 from rasa.shared.core.generator import TrainingDataGenerator
 from rasa.shared.core.training_data.structures import StoryGraph, StoryStep
 from rasa.shared.nlu.constants import (
@@ -15,7 +15,6 @@ from rasa.shared.nlu.constants import (
     INTENT,
     TEXT,
     ENTITY_ATTRIBUTE_TYPE,
-    ENTITIES,
     INTENT_NAME_KEY,
 )
 
@@ -59,25 +58,15 @@ class UserMessageGenerator:
         ) != e.get(ENTITY_ATTRIBUTE_VALUE)
 
     def message_for_data(self, structured_info: Dict[Text, Any]) -> Any:
-        """Find a data sample with the same intent and entities.
-
-        Given the parsed data from a message (intent and entities) finds a
-        message in the data that has the same intent and entities."""
-
+        """Find a data sample with the same intent."""
         if structured_info.get(INTENT) is not None:
             intent_name = structured_info.get(INTENT, {}).get(INTENT_NAME_KEY)
             usable_examples = self.mapping.get(intent_name, [])[:]
             random.shuffle(usable_examples)
-            for example in usable_examples:
-                entities = {
-                    e.get(ENTITY_ATTRIBUTE_TYPE): e.get(ENTITY_ATTRIBUTE_VALUE)
-                    for e in example.get(ENTITIES, [])
-                }
-                for e in structured_info.get(ENTITIES, []):
-                    if self._contains_same_entity(entities, e):
-                        break
-                else:
-                    return example.get(TEXT)
+
+            if usable_examples:
+                return usable_examples[0].get(TEXT)
+
         return structured_info.get(TEXT)
 
 
@@ -266,13 +255,12 @@ def _merge_equivalent_nodes(graph: "networkx.MultiDiGraph", max_history: int) ->
                         graph.remove_node(j)
 
 
-async def _replace_edge_labels_with_nodes(
-    graph: "networkx.MultiDiGraph",
-    next_id: int,
-    interpreter: NaturalLanguageInterpreter,
-    nlu_training_data: "TrainingData",
+def _replace_edge_labels_with_nodes(
+    graph: "networkx.MultiDiGraph", next_id: int, nlu_training_data: "TrainingData",
 ) -> None:
-    """User messages are created as edge labels. This removes the labels and
+    """Replaces edge labels with nodes.
+
+    User messages are created as edge labels. This removes the labels and
     creates nodes instead.
 
     The algorithms (e.g. merging) are simpler if the user messages are labels
@@ -288,11 +276,14 @@ async def _replace_edge_labels_with_nodes(
     edges = list(graph.edges(keys=True, data=True))
     for s, e, k, d in edges:
         if k != EDGE_NONE_LABEL:
-            if message_generator and d.get("label", k) is not None:
-                parsed_info = await interpreter.parse(d.get("label", k))
+            label = d.get("label", k)
+
+            if message_generator:
+                parsed_info = {TEXT: label}
+                if label.startswith(INTENT_MESSAGE_PREFIX):
+                    parsed_info[INTENT] = {INTENT_NAME_KEY: label[1:]}
+
                 label = message_generator.message_for_data(parsed_info)
-            else:
-                label = d.get("label", k)
             next_id += 1
             graph.remove_edge(s, e, k)
             graph.add_node(
@@ -412,12 +403,11 @@ def _add_message_edge(
     )
 
 
-async def visualize_neighborhood(
+def visualize_neighborhood(
     current: Optional[List[Event]],
     event_sequences: List[List[Event]],
     output_file: Optional[Text] = None,
     max_history: int = 2,
-    interpreter: NaturalLanguageInterpreter = RegexInterpreter(),
     nlu_training_data: Optional["TrainingData"] = None,
     should_merge_nodes: bool = True,
     max_distance: int = 1,
@@ -447,10 +437,8 @@ async def visualize_neighborhood(
                 idx -= 1
                 break
             if isinstance(el, UserUttered):
-                if not el.intent:
-                    message = await interpreter.parse(el.text)
-                else:
-                    message = el.parse_data
+                message = el.parse_data
+                message[TEXT] = f"{INTENT_MESSAGE_PREFIX}{el.intent_name}"
             elif (
                 isinstance(el, ActionExecuted) and el.action_name != ACTION_LISTEN_NAME
             ):
@@ -508,9 +496,7 @@ async def visualize_neighborhood(
 
     if should_merge_nodes:
         _merge_equivalent_nodes(graph, max_history)
-    await _replace_edge_labels_with_nodes(
-        graph, next_node_idx, interpreter, nlu_training_data
-    )
+    _replace_edge_labels_with_nodes(graph, next_node_idx, nlu_training_data)
 
     _remove_auxiliary_nodes(graph, special_node_idx)
 
@@ -539,12 +525,11 @@ def _remove_auxiliary_nodes(
                 ps.add(pred)
 
 
-async def visualize_stories(
+def visualize_stories(
     story_steps: List[StoryStep],
     domain: Domain,
     output_file: Optional[Text],
     max_history: int,
-    interpreter: NaturalLanguageInterpreter = RegexInterpreter(),
     nlu_training_data: Optional["TrainingData"] = None,
     should_merge_nodes: bool = True,
     fontsize: int = 12,
@@ -588,12 +573,11 @@ async def visualize_stories(
     completed_trackers = g.generate()
     event_sequences = [t.events for t in completed_trackers]
 
-    graph = await visualize_neighborhood(
+    graph = visualize_neighborhood(
         None,
         event_sequences,
         output_file,
         max_history,
-        interpreter,
         nlu_training_data,
         should_merge_nodes,
         max_distance=1,
