@@ -1,4 +1,4 @@
-from typing import Iterable, Text, Optional, Dict, List
+from typing import Iterable, Text, Optional, List
 
 from rasa.shared.core.domain import Domain
 from rasa.shared.core.training_data.structures import StoryGraph
@@ -12,17 +12,11 @@ def training_data_from_paths(paths: Iterable[Text], language: Text) -> TrainingD
     return TrainingData().merge(*training_data_sets)
 
 
-async def story_graph_from_paths(
-    files: List[Text],
-    domain: Domain,
-    template_variables: Optional[Dict] = None,
-    use_e2e: bool = False,
-    exclusion_percentage: Optional[int] = None,
+def story_graph_from_paths(
+    files: List[Text], domain: Domain, exclusion_percentage: Optional[int] = None,
 ) -> StoryGraph:
-
+    """Returns the `StoryGraph` from paths."""
     from rasa.shared.core.training_data import loading
 
-    story_steps = await loading.load_data_from_files(
-        files, domain, template_variables, use_e2e, exclusion_percentage
-    )
+    story_steps = loading.load_data_from_files(files, domain, exclusion_percentage)
     return StoryGraph(story_steps)
