@@ -9,7 +9,8 @@ from pathlib import Path
 import re
 from typing import Any, Dict, List, Optional, Text, Type, Union
 import warnings
-import logging
+import random
+import string
 
 from ruamel import yaml as yaml
 from ruamel.yaml import RoundTripRepresenter, YAMLError
@@ -21,8 +22,6 @@ from rasa.shared.constants import (
     NEXT_MAJOR_VERSION_FOR_DEPRECATIONS,
     CONFIG_SCHEMA_FILE,
     MODEL_CONFIG_SCHEMA_FILE,
-    DEFAULT_YAML_SEQUENCE_INDENT,
-    DEFAULT_YAML_SEQUENCE_INDENT_ENV_VAR,
 )
 from rasa.shared.exceptions import (
     FileIOException,
@@ -32,8 +31,6 @@ from rasa.shared.exceptions import (
 )
 import rasa.shared.utils.validation
 
-logger = logging.getLogger(__name__)
-
 DEFAULT_ENCODING = "utf-8"
 YAML_VERSION = (1, 2)
 
@@ -204,7 +201,7 @@ def list_subdirectories(path: Text) -> List[Text]:
 
 
 def deep_container_fingerprint(
-    obj: Union[List[Any], Dict[Any, Any]], encoding: Text = DEFAULT_ENCODING
+    obj: Union[List[Any], Dict[Any, Any], Any], encoding: Text = DEFAULT_ENCODING
 ) -> Text:
     """Calculate a hash which is stable, independent of a containers key order.
 
@@ -221,8 +218,10 @@ def deep_container_fingerprint(
     """
     if isinstance(obj, dict):
         return get_dictionary_fingerprint(obj, encoding)
-    if isinstance(obj, list):
+    elif isinstance(obj, list):
         return get_list_fingerprint(obj, encoding)
+    elif hasattr(obj, "fingerprint") and callable(obj.fingerprint):
+        return obj.fingerprint()
     else:
         return get_text_hash(str(obj), encoding)
 
@@ -400,22 +399,6 @@ def write_yaml(
         data = convert_to_ordered_dict(data)
 
     dumper = yaml.YAML()
-
-    sequence_indent = int(
-        os.environ.get(
-            DEFAULT_YAML_SEQUENCE_INDENT_ENV_VAR, DEFAULT_YAML_SEQUENCE_INDENT
-        )
-    )
-    if sequence_indent < 2:
-        logger.warning(
-            f"The value of the env var {DEFAULT_YAML_SEQUENCE_INDENT_ENV_VAR} "
-            f"is {sequence_indent}. Values less than 2 are not supported - see "
-            f"https://yaml.readthedocs.io/en/latest/detail.html#indentation-of-block-sequences. "  # noqa: E501, W505
-            f"YAML will be dumped using a sequence indent value of "
-            f"{DEFAULT_YAML_SEQUENCE_INDENT} instead."
-        )
-        sequence_indent = DEFAULT_YAML_SEQUENCE_INDENT
-    dumper.indent(mapping=2, sequence=sequence_indent, offset=sequence_indent - 2)
     # no wrap lines
     dumper.width = YAML_LINE_MAX_WIDTH
 
@@ -639,3 +622,8 @@ def is_subdirectory(path: Text, potential_parent_directory: Text) -> bool:
     potential_parent_directory = os.path.abspath(potential_parent_directory)
 
     return potential_parent_directory in path
+
+
+def random_string(length: int) -> Text:
+    """Returns a random string of given length."""
+    return "".join(random.choices(string.ascii_uppercase + string.digits, k=length))
