@@ -1,28 +1,27 @@
+import itertools
 import logging
 from collections import defaultdict
-from typing import Set, Text, Optional, Dict, Any
+from typing import Set, Text, Optional, Dict, Any, List
 
 import rasa.core.training.story_conflict
 import rasa.shared.nlu.constants
 from rasa.shared.constants import (
     DOCS_URL_DOMAINS,
+    DOCS_URL_FORMS,
     UTTER_PREFIX,
     DOCS_URL_ACTIONS,
+    REQUIRED_SLOTS_KEY,
 )
-from rasa.shared.core.domain import (
-    KEY_INTENTS,
-    KEY_RESPONSES,
-    KEY_SLOTS,
-    KEY_FORMS,
-    Domain,
-)
-from rasa.shared.core.events import ActionExecuted
+from rasa.shared.core import constants
+from rasa.shared.core.constants import MAPPING_CONDITIONS, ACTIVE_LOOP
+from rasa.shared.core.domain import Domain
+from rasa.shared.core.events import ActionExecuted, ActiveLoop
 from rasa.shared.core.events import UserUttered
 from rasa.shared.core.generator import TrainingDataGenerator
+from rasa.shared.core.slot_mappings import SlotMapping
 from rasa.shared.core.training_data.structures import StoryGraph
 from rasa.shared.importers.importer import TrainingDataImporter
 from rasa.shared.nlu.training_data.training_data import TrainingData
-from rasa.nlu.config import RasaNLUModelConfig
 import rasa.shared.utils.io
 
 logger = logging.getLogger(__name__)
@@ -49,27 +48,33 @@ class Validator:
         self.domain = domain
         self.intents = intents
         self.story_graph = story_graph
-        self.nlu_config = RasaNLUModelConfig(config)
 
     @classmethod
-    async def from_importer(cls, importer: TrainingDataImporter) -> "Validator":
+    def from_importer(cls, importer: TrainingDataImporter) -> "Validator":
         """Create an instance from the domain, nlu and story files."""
-        domain = await importer.get_domain()
-        story_graph = await importer.get_stories()
-        intents = await importer.get_nlu_data()
-        config = await importer.get_config()
+        domain = importer.get_domain()
+        story_graph = importer.get_stories()
+        intents = importer.get_nlu_data()
+        config = importer.get_config()
 
         return cls(domain, intents, story_graph, config)
 
+    def _non_default_intents(self) -> List[Text]:
+        return [
+            item
+            for item in self.domain.intents
+            if item not in constants.DEFAULT_INTENTS
+        ]
+
     def verify_intents(self, ignore_warnings: bool = True) -> bool:
         """Compares list of intents in domain with intents in NLU training data."""
         everything_is_alright = True
 
         nlu_data_intents = {e.data["intent"] for e in self.intents.intent_examples}
 
-        for intent in self.domain.intents:
+        for intent in self._non_default_intents():
             if intent not in nlu_data_intents:
-                logger.debug(
+                rasa.shared.utils.io.raise_warning(
                     f"The intent '{intent}' is listed in the domain file, but "
                     f"is not found in the NLU training data."
                 )
@@ -137,9 +142,11 @@ class Validator:
                 )
                 everything_is_alright = False
 
-        for intent in self.domain.intents:
+        for intent in self._non_default_intents():
             if intent not in stories_intents:
-                logger.debug(f"The intent '{intent}' is not used in any story.")
+                rasa.shared.utils.io.raise_warning(
+                    f"The intent '{intent}' is not used in any story or rule."
+                )
                 everything_is_alright = ignore_warnings and everything_is_alright
 
         return everything_is_alright
@@ -198,11 +205,40 @@ class Validator:
 
         for utterance in utterance_actions:
             if utterance not in stories_utterances:
-                logger.debug(f"The utterance '{utterance}' is not used in any story.")
+                rasa.shared.utils.io.raise_warning(
+                    f"The utterance '{utterance}' is not used in any story or rule."
+                )
                 everything_is_alright = ignore_warnings and everything_is_alright
 
         return everything_is_alright
 
+    def verify_forms_in_stories_rules(self) -> bool:
+        """Verifies that forms referenced in active_loop directives are present."""
+        all_forms_exist = True
+        visited_loops = set()
+
+        for story in self.story_graph.story_steps:
+            for event in story.events:
+                if not isinstance(event, ActiveLoop):
+                    continue
+
+                if event.name in visited_loops:
+                    # We've seen this loop before, don't alert on it twice
+                    continue
+
+                if event.name not in self.domain.form_names:
+                    rasa.shared.utils.io.raise_warning(
+                        f"The form '{event.name}' is used in the "
+                        f"'{story.block_name}' block, but it "
+                        f"is not listed in the domain file. You should add it to your "
+                        f"domain file!",
+                        docs=DOCS_URL_FORMS,
+                    )
+                    all_forms_exist = False
+                visited_loops.add(event.name)
+
+        return all_forms_exist
+
     def verify_actions_in_stories_rules(self) -> bool:
         """Verifies that actions used in stories and rules are present in the domain."""
         everything_is_alright = True
@@ -259,7 +295,7 @@ class Validator:
 
         # Create a list of `StoryConflict` objects
         conflicts = rasa.core.training.story_conflict.find_story_conflicts(
-            trackers, self.domain, max_history, self.nlu_config
+            trackers, self.domain, max_history,
         )
 
         if not conflicts:
@@ -291,8 +327,8 @@ class Validator:
         everything_is_alright = True
 
         for form in self.domain.form_names:
-            form_slots = self.domain.slot_mapping_for_form(form)
-            for slot in form_slots.keys():
+            form_slots = self.domain.required_slots_for_form(form)
+            for slot in form_slots:
                 if slot in domain_slot_names:
                     continue
                 else:
@@ -306,6 +342,56 @@ class Validator:
 
         return everything_is_alright
 
+    def verify_slot_mappings(self) -> bool:
+        """Verifies that slot mappings match forms."""
+        everything_is_alright = True
+
+        all_form_slots = [
+            required_slots[REQUIRED_SLOTS_KEY]
+            for required_slots in self.domain.forms.values()
+        ]
+        all_required_slots = set(itertools.chain.from_iterable(all_form_slots))
+
+        for slot in self.domain.slots:
+            for mapping in slot.mappings:
+                for condition in mapping.get(MAPPING_CONDITIONS, []):
+                    condition_active_loop = condition.get(ACTIVE_LOOP)
+                    if (
+                        condition_active_loop
+                        and condition_active_loop not in self.domain.form_names
+                    ):
+                        rasa.shared.utils.io.raise_warning(
+                            f"Slot '{slot.name}' has a mapping condition for form "
+                            f"'{condition_active_loop}' which is not listed in "
+                            f"domain forms. Please add this form to the forms section "
+                            f"or check for typos."
+                        )
+                        everything_is_alright = False
+
+                    form_slots = self.domain.forms.get(condition_active_loop, {}).get(
+                        REQUIRED_SLOTS_KEY, {}
+                    )
+                    if form_slots and slot.name not in form_slots:
+                        rasa.shared.utils.io.raise_warning(
+                            f"Slot '{slot.name}' has a mapping condition for form "
+                            f"'{condition_active_loop}', but it's not present in "
+                            f"'{condition_active_loop}' form's '{REQUIRED_SLOTS_KEY}'. "
+                            f"The slot needs to be added to this key."
+                        )
+                        everything_is_alright = False
+
+                if (
+                    mapping.get("type") == str(SlotMapping.FROM_TRIGGER_INTENT)
+                    and slot.name not in all_required_slots
+                ):
+                    rasa.shared.utils.io.raise_warning(
+                        f"Slot '{slot.name}' has a 'from_trigger_intent' mapping, "
+                        f"but it's not listed in any form '{REQUIRED_SLOTS_KEY}'."
+                    )
+                    everything_is_alright = False
+
+        return everything_is_alright
+
     def verify_domain_validity(self) -> bool:
         """Checks whether the domain returned by the importer is empty.
 
@@ -325,29 +411,3 @@ class Validator:
                 return False
 
         return True
-
-    def verify_domain_duplicates(self) -> bool:
-        """Verifies that there are no duplicated dictionaries in multiple domain files.
-
-        Returns:
-            `True` if duplicates exist.
-        """
-        logger.info("Checking duplicates across domain files...")
-
-        all_valid = True
-
-        if not self.domain.duplicates:
-            return True
-
-        for key in [KEY_INTENTS, KEY_FORMS, KEY_RESPONSES, KEY_SLOTS]:
-            duplicates = self.domain.duplicates.get(key)
-            if duplicates:
-                duplicates_str = ", ".join(duplicates)
-                rasa.shared.utils.io.raise_warning(
-                    f"The following duplicated {key} has been found "
-                    + f"across multiple domain files: {duplicates_str}",
-                    docs=DOCS_URL_DOMAINS,
-                )
-                all_valid = False
-
-        return all_valid
